/*
 *  Copyright (C) Chaperon. All rights reserved.
 *  -------------------------------------------------------------------------
 *  This software is published under the terms of the Apache Software License
 *  version 1.1, a copy of which has been included  with this distribution in
 *  the LICENSE file.
 */

package net.sourceforge.chaperon.grammar.symbol;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.ObjectStreamException;
import java.io.Serializable;
import java.util.Enumeration;
import java.util.Hashtable;

/**
 * This class represent a empty symbol
 *
 * @version $Revision: 1.1 $ $Date: 2002/04/09 21:46:17 $
 */
public class EmptySymbol implements Symbol, Serializable
{

  /**
   * The instance of the symbol
   */
  public static final EmptySymbol EMPTY = new EmptySymbol("empty");

	private static Hashtable _memberTable = init();

	private String _name = null;

	/**
	 * The private constructor for the symbol
	 *
	 * @param type Integer type      
	 * @param value String type
	 */
	private EmptySymbol(String name)
	{
		super();
		_name = name;
	}
  
  private static java.util.Hashtable init()
  {
    Hashtable members = new Hashtable();
  
    members.put("empty", EMPTY);
    return members;
  }

	/**
	 * Returns an enumeration of all possible instances of this symbol
	 */
	public static Enumeration enumerate()
	{
		return _memberTable.elements();
	}

  /**
   * Returns the name of this symbol
   *
   * @return Name of symbol
   */
  public String getName()
  {
    return _name;
  }

  /**
   * Indicates whenever his is symbol is a terminal symbol
   *
   * @return True, if this symbol is a terminal symbol
   */
  public boolean isTerminal()
  {
    return false;
  }
  
  /**
   * Indicates whenever his is symbol is a non terminal symbol
   *
   * @return True, if this symbol is a non terminal symbol
   */
  public boolean isNonTerminal()
  {
    return false;
  }
  
  /**
   * Indicates whenever his is symbol is a symbol
   * for an empty list
   *
   * @return True, if this symbol indicates an empty list
   */
  public boolean isEmpty()
  {
    return true;
  }
    
	/**
	 * Returns the String representation of this empty symbol
	 */
	public String toString()
	{
		return _name;
	}

	/**
	 * Returns a new symbol based on the given name.
	 * @param name Name
	 */
	public static EmptySymbol valueOf(String name)
	{
		Object obj = null;

		if (name != null)
			obj = _memberTable.get(name);
		if (obj == null)
		{
      obj = new EmptySymbol(name);
      _memberTable.put(name, obj);
		}
		return (EmptySymbol) obj;
	}

  private void writeObject(ObjectOutputStream out) throws IOException
  {
    out.defaultWriteObject();
    out.writeUTF(_name);
  }

  private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException
  {
    in.defaultReadObject();
    _name = in.readUTF();
  }

  /** 
   * Deals with the issue of uniqueness when we are dealing with more
   * than one VM by adding the read symbol to the hash table, if it
   * isn't already there.
   */
  public Object readResolve() throws ObjectStreamException
  {
    return EMPTY;
  }
}
